require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/chaseandattack"
require "behaviours/panic"
require "behaviours/follow"
require "behaviours/attackwall"

local Arm_WebberBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

--Images will help chop, mine and fight.

local MIN_FOLLOW_DIST = 2
local TARGET_FOLLOW_DIST = 3
local MAX_FOLLOW_DIST = 6
local GO_HOME_DIST = 0

local START_FACE_DIST =6
local KEEP_FACE_DIST = 8

local KEEP_WORKING_DIST = 15
local SEE_WORK_DIST = 20

local function HasStateTags(inst, tags)
    for k,v in pairs(tags) do
        if inst.sg:HasStateTag(v) then
            return true
        end
    end
end


function Arm_WebberBrain:CanSpawnChild()
	
	local player = GetPlayer()
	
	return self.inst:GetTimeAlive() > 1 and 
		self.inst.components.leader.numfollowers < 4 and
		 (self.inst.components.combat.target or self.inst:GetDistanceSqToInst(player) < TUNING.ZHUISHAJULI)

end

local function KeepWorkingAction(inst, actiontags)
    return inst.components.follower.leader and inst.components.follower.leader:GetDistanceSqToInst(inst) <= KEEP_WORKING_DIST*KEEP_WORKING_DIST and 
    HasStateTags(inst.components.follower.leader, actiontags)
end

local function StartWorkingCondition(inst, actiontags)
    return inst.components.follower.leader and HasStateTags(inst.components.follower.leader, actiontags) and not HasStateTags(inst, actiontags)
end

local function FindObjectToWorkAction(inst, action)
    if inst.sg:HasStateTag("working") then
        return 
    end
    local target = FindEntity(inst.components.follower.leader, SEE_WORK_DIST, function(item) return item.components.workable and item.components.workable.action == action end)
    if target then
        --print(GetTime(), target)
        return BufferedAction(inst, target, action)
    end
end

--[[local function ShouldStopCommand(inst, target)
    return inst.stop 
end]]

local function GetLeader(inst)
    return inst.components.follower.leader 
end

local function GetFaceTargetFn(inst)
    local target = GetClosestInstWithTag("player", inst, START_FACE_DIST)
    if target and not target:HasTag("notarget") then
        return target
    end
end

local function KeepFaceTargetFn(inst, target)
    return inst:IsNear(target, KEEP_FACE_DIST) and not target:HasTag("notarget")
end

local function HideRunAway(inst, target)
    return inst.hide
end

function Arm_WebberBrain:OnStart()
    local root = PriorityNode(
    {			
	
			WhileNode( function() return self.inst.components.health:GetPercent() < 1 and self.inst.components.combat.target == nil or not self.inst.components.combat:InCooldown() end, "AttackMomentarily", ChaseAndAttack(self.inst, 8,40)),			
			WhileNode( function() return self.inst.components.health:GetPercent() < 1 and self.inst.components.combat.target and self.inst.components.combat:InCooldown() end, "Dodge", ChaseAndAttack(self.inst, 8,TUNING.ZHUISHAJULI)),
			--召唤
			MinPeriod(self.inst, 20, --召唤时间
			IfNode(function() return self:CanSpawnChild() end, "needs follower", 
				ActionNode(function() self.inst.sg:GoToState("summon_perd") return SUCCESS end, "make child" ))),	
			
			
			
			
			
			RunAway(self.inst, "player", 1, 6, function(target) return HideRunAway(self.inst, target) end ),
        --[[  
        WhileNode(function() return StartWorkingCondition(self.inst, {"chopping", "prechop"}) and 
        KeepWorkingAction(self.inst, {"chopping", "prechop"}) end, "keep chopping",
            DoAction(self.inst, function() return FindObjectToWorkAction(self.inst, ACTIONS.CHOP) end)),

        WhileNode(function() return StartWorkingCondition(self.inst, {"mining", "premine"}) and 
        KeepWorkingAction(self.inst, {"mining", "premine"}) end, "keep mining",                   
            DoAction(self.inst, function() return FindObjectToWorkAction(self.inst, ACTIONS.MINE) end)),


        Follow(self.inst, GetLeader, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
        IfNode(function() return GetLeader(self.inst) end, "has leader",            
            FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn )),
]]
 --RunAway(self.inst, "hiding", 4, 6),
 
 
 
 Wander(self.inst, function() return Vector3(self.inst.Transform:GetWorldPosition()) end, MAX_FOLLOW_DIST- 1, {minwalktime=3, randwalktime=3, minwaittime=6, randwaittime=3}),
    }, 0)
    

    self.bt = BT(self.inst, root)    
end

return Arm_WebberBrain