local trace = function() end

local assets=
{
	Asset("ANIM", "anim/dragonfly_fx.zip"),
	Asset("ANIM", "anim/dragoon_build.zip"),
	Asset("ANIM", "anim/dragoon_basic.zip"),
	Asset("ANIM", "anim/dragoon_actions.zip"),
	Asset( "IMAGE", "images/map_icons/arm_bro2.tex" ),
	Asset( "ATLAS", "images/map_icons/arm_bro2.xml" ),
}

local prefabs =
{
	"monstermeat",
	"firesplash_fx",
	"firering_fx",
	"dragoonfire",
	"dragonfly_fx",
	"dragoonspit",
	"dragoonheart",
	"dragoon_charge_fx",
}



local WAKE_TO_FOLLOW_DISTANCE = 8
local SHARE_TARGET_DIST = TUNING.DRAGOON_KEEP_TARGET_DIST

local NO_TAGS = {"FX", "NOCLICK","DECOR","INLIMBO"}


local function assasin_hunt(inst)
    --local dist = 40
    return FindEntity(inst, TUNING.ZHUISHAJULI, function(guy)
            return ( guy:HasTag("player") or guy:HasTag("yamche"))
			and not guy:HasTag("toso_arm")
			 and guy.components.health
                and not guy.components.health:IsDead()
                and inst.components.combat:CanTarget(guy)
        end)
		end

local function Onhunt(inst, target)
    return inst.components.combat:CanTarget(target) and not target:HasTag("toso_arm")
end

local function ShouldWakeUp(inst)
	return DefaultWakeTest(inst)
end

local function ShouldSleep(inst)
	return DefaultSleepTest(inst)
end 

local function OnNewTarget(inst, data)
	if inst.components.sleeper:IsAsleep() then
		inst.components.sleeper:WakeUp()
	end
end

local function retargetfn(inst)
	return FindEntity(inst, TUNING.DRAGOON_TARGET_DIST, 
		function(guy) 
			return inst.components.combat:CanTarget(guy)
		end, 
		nil, {"wall", "dragoon", "elephantcactus", "FX", "NOCLICK"})
end

local function KeepTarget(inst, target)
	return inst.components.combat:CanTarget(target) and inst:GetDistanceSqToInst(target) <= (TUNING.DRAGOON_KEEP_TARGET_DIST*TUNING.DRAGOON_KEEP_TARGET_DIST)
end

local function OnAttacked(inst, data)
	inst.components.combat:SetTarget(data.attacker)
	inst.components.combat:ShareTarget(data.attacker, SHARE_TARGET_DIST, function(dude) return dude:HasTag("dragoon") and not dude.components.health:IsDead() end, 5)
end

local function OnAttackOther(inst, data)
	inst.components.combat:ShareTarget(data.target, SHARE_TARGET_DIST, function(dude) return dude:HasTag("dragoon") and not dude.components.health:IsDead() end, 5)
end

local function GetReturnPos(inst)
	local rad = 2
	local pos = inst:GetPosition()
	trace("GetReturnPos", inst, pos)
	local angle = math.random()*2*PI
	pos = pos + Point(rad*math.cos(angle), 0, -rad*math.sin(angle))
	trace("    ", pos)
	return pos:Get()
end

local function DoReturn(inst)
	--print("DoReturn", inst)
	if inst.components.homeseeker and inst.components.homeseeker:HasHome()  then
		inst.components.homeseeker.home.components.childspawner:GoHome(inst)
	end
end

local function OnNight(inst)
	--print("OnNight", inst)
	if inst:IsAsleep() then
		DoReturn(inst)  
	end
end


local function OnEntitySleep(inst)
	--print("OnEntitySleep", inst)
	if not GetClock():IsDay() then
		DoReturn(inst)
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local physics = inst.entity:AddPhysics()
	local sound = inst.entity:AddSoundEmitter()
	
	local s = 1.3
	inst.Transform:SetScale(s,s,s)

	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize(3, 1.25)
	
	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon( "bunnyman.tex" )

	inst.Transform:SetFourFaced()
	
	inst:AddTag("scarytoprey")
	inst:AddTag("monster")
	inst:AddTag("hostile")
	inst:AddTag("lavaspitter")
	inst:AddTag("dragoon")
    inst:AddTag("groundpoundimmune")
	--所有敌对人形怪
	inst:AddTag("toso_arm")
	
	MakePoisonableCharacter(inst)
	MakeCharacterPhysics(inst, 0, .5)

	inst.last_spit_time = nil
	inst.last_target_spit_time = nil
	inst.spit_interval = math.random(20,30)
	inst.num_targets_vomited = 0
	 
	anim:SetBank("dragoon")
	anim:SetBuild("dragoon_build")
	anim:PlayAnimation("idle_loop")
	inst.AnimState:SetRayTestOnBB(true)

	inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
	inst.components.locomotor.walkspeed = 5
	inst.components.locomotor.runspeed = 7
	inst:SetStateGraph("SGdragoon")

	local brain = require "brains/dragoonbrain"
	inst:SetBrain(brain)

local player = GetPlayer()
	
	inst:AddComponent("health")
	--生命护甲数值
	inst.components.health:SetMaxHealth(800+player.level*TUNING.ARM_HEALTHUP)
    inst.components.health:SetAbsorptionAmount(TUNING.ARM_ARMOR)
	
	
	inst.components.health.fire_damage_scale = 0


	
	inst:AddComponent("combat")
	--攻速攻击力数值
	 inst.components.combat:SetDefaultDamage(80+player.level*5)
	inst.components.combat:SetAttackPeriod(TUNING.ARM_ATKSPEED*0.5)
	
	
    inst.components.combat:SetRetargetFunction(0, assasin_hunt)
	inst.components.combat:SetKeepTargetFunction(Onhunt)
	inst.components.combat:SetHurtSound("dontstarve_DLC002/creatures/dragoon/hit")
	inst.components.combat:SetRange(2,2)

	inst:AddComponent("lootdropper")
	inst.components.lootdropper:AddLoot("monstermeat")
	inst.components.lootdropper:AddChanceLoot("dragoonheart", .1)

	
	inst:AddComponent("inspectable")
	
	inst:AddComponent("sleeper")
	inst.components.sleeper.testperiod = GetRandomWithVariance(6, 2)
	inst.components.sleeper:SetSleepTest(ShouldSleep)
	inst.components.sleeper:SetWakeTest(ShouldWakeUp)
	inst:ListenForEvent("newcombattarget", OnNewTarget)

	inst:ListenForEvent( "dusktime", function() OnNight( inst ) end, GetWorld()) 
	inst:ListenForEvent( "nighttime", function() OnNight( inst ) end, GetWorld()) 
	inst.OnEntitySleep = OnEntitySleep

	inst:ListenForEvent("attacked", OnAttacked)
	inst:ListenForEvent("onattackother", OnAttackOther)

	inst:DoPeriodicTask(900, function() 
	inst:DoTaskInTime(1.5, function()  SpawnPrefab("lightning").Transform:SetPosition(inst:GetPosition():Get())
	--inst:DoTaskInTime(0.25, function() SpawnPrefab("lightning").Transform:SetPosition(inst:GetPosition():Get()) 
	
	inst:DoTaskInTime(10, function() 
	--SpawnPrefab("lightning").Transform:SetPosition(inst:GetPosition():Get())
	local player = GetPlayer()	
	        local downvec = TheCamera:GetDownVec()
            local x,y,z = player.Transform:GetWorldPosition()	
            --local arm2 = inst
			local theta = math.random() * 2 * PI
			local radius = 25
            local offset = FindWalkableOffset(inst:GetPosition(), theta, radius, 12, true) or FindWaterOffset(inst:GetPosition(), theta, radius, 36, false) or FindWaterOffset(inst:GetPosition(), theta, radius, 36, false) or downvec:Normalize()
            inst.Transform:SetPosition(x + offset.x, y + offset.y, z + offset.z)
			inst.SoundEmitter:PlaySound("dontstarve/common/craftable/lightning_rod")
            SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get()) 			
	
	end) 
	end) 
	end)

	MakeMediumFreezableCharacter(inst, "hound_body")
	MakeLargePropagator(inst)
	inst.components.propagator.decayrate = 0

	return inst
end

return Prefab("monsters/arm_dragoon", fn, assets, prefabs)
