require "brains/houndbrain"
require "stategraphs/SGboss_hound"

local trace = function() end

local assets=
{
	Asset("ANIM", "anim/hound_basic.zip"),
	Asset("ANIM", "anim/hound.zip"),
	Asset("ANIM", "anim/hound_red.zip"),
	Asset("ANIM", "anim/hound_ice.zip"),
	Asset("SOUND", "sound/hound.fsb"),
}

local prefabs =
{
	"houndstooth",
	"monstermeat",
	"redgem",
	"bluegem",
}

SetSharedLootTable( 'hound',
{
    {'monstermeat', 1.000},
    {'houndstooth',  0.125},
})
 
SetSharedLootTable( 'hound_fire',
{
    {'monstermeat', 1.0},
    {'houndstooth', 1.0},
    {'houndfire',   1.0},
    {'houndfire',   1.0},
    {'houndfire',   1.0},
    {'redgem',      0.2},
})

SetSharedLootTable( 'hound_cold',
{
    {'monstermeat', 1.0},
    {'houndstooth', 1.0},
    {'houndstooth', 1.0},
    {'bluegem',     0.2},
})

local WAKE_TO_FOLLOW_DISTANCE = 8
local SLEEP_NEAR_HOME_DISTANCE = 10
local SHARE_TARGET_DIST = 30
local HOME_TELEPORT_DIST = 30

local NO_TAGS = {"FX", "NOCLICK","DECOR","INLIMBO"}

local function houndatk(inst,data)
local v = data.target
if v and v:HasTag("player") then
v.sg:GoToState("hit")

end

end

local function assasin_hunt(inst)
    --local dist = 40
    return FindEntity(inst, TUNING.ZHUISHAJULI, function(guy)
            return ( guy:HasTag("player") or guy:HasTag("yamche"))
			and not guy:HasTag("toso_arm")
			 and guy.components.health
                and not guy.components.health:IsDead()
                and inst.components.combat:CanTarget(guy)
        end)
		end

local function Onhunt(inst, target)
    return inst.components.combat:CanTarget(target) and not target:HasTag("toso_arm")
end


local function ShouldWakeUp(inst)
    return DefaultWakeTest(inst) or (inst.components.follower and inst.components.follower.leader and not inst.components.follower:IsNearLeader(WAKE_TO_FOLLOW_DISTANCE))
end

local function ShouldSleep(inst)
    return inst:HasTag("pet_hound")
    and not GetClock():IsDay()
    and not (inst.components.combat and inst.components.combat.target)
    and not (inst.components.burnable and inst.components.burnable:IsBurning() )
    and (not inst.components.homeseeker or inst:IsNear(inst.components.homeseeker.home, SLEEP_NEAR_HOME_DISTANCE))
end

local function OnNewTarget(inst, data)
    if inst.components.sleeper:IsAsleep() then
        inst.components.sleeper:WakeUp()
    end
end


local function retargetfn(inst)
    local dist = TUNING.HOUND_TARGET_DIST
    if inst:HasTag("pet_hound") then
        dist = TUNING.HOUND_FOLLOWER_TARGET_DIST
    end
    return FindEntity(inst, dist, function(guy) 
		return not guy:HasTag("wall") and not guy:HasTag("houndmound") and not (guy:HasTag("hound") or guy:HasTag("houndfriend")) and inst.components.combat:CanTarget(guy)
    end)
end

local function KeepTarget(inst, target)
    return inst.components.combat:CanTarget(target) and (not inst:HasTag("pet_hound") or inst:IsNear(target, TUNING.HOUND_FOLLOWER_TARGET_KEEP))
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, SHARE_TARGET_DIST, function(dude) return dude:HasTag("hound") or dude:HasTag("houndfriend") and not dude.components.health:IsDead() end, 5)
end

local function OnAttackOther(inst, data)
    inst.components.combat:ShareTarget(data.target, SHARE_TARGET_DIST, function(dude) return dude:HasTag("hound") or dude:HasTag("houndfriend") and not dude.components.health:IsDead() end, 5)
end

local function GetReturnPos(inst)
    local rad = 2
    local pos = inst:GetPosition()
    trace("GetReturnPos", inst, pos)
    local angle = math.random()*2*PI
    pos = pos + Point(rad*math.cos(angle), 0, -rad*math.sin(angle))
    trace("    ", pos)
    return pos:Get()
end

local function DoReturn(inst)
    --print("DoReturn", inst)
    if inst.components.homeseeker and inst.components.homeseeker:HasHome()  then
        if inst:HasTag("pet_hound") then
            if inst.components.homeseeker.home:IsAsleep() and not inst:IsNear(inst.components.homeseeker.home, HOME_TELEPORT_DIST) then
                local x, y, z = GetReturnPos(inst.components.homeseeker.home)
                inst.Physics:Teleport(x, y, z)
                trace("hound warped home", x, y, z)
            end
        elseif inst.components.homeseeker.home.components.childspawner then
            inst.components.homeseeker.home.components.childspawner:GoHome(inst)
        end
    end
end

local function OnNight(inst)
    --print("OnNight", inst)
    if inst:IsAsleep() then
        DoReturn(inst)  
    end
end


local function OnEntitySleep(inst)
    --print("OnEntitySleep", inst)
    if not GetClock():IsDay() then
        DoReturn(inst)
    end
end

local function OnSave(inst, data)
    data.ispet = inst:HasTag("pet_hound")
    --print("OnSave", inst, data.ispet)
end
        
local function OnLoad(inst, data)
    --print("OnLoad", inst, data.ispet)
    if data and data.ispet then
        inst:AddTag("pet_hound")

        if inst.sg then
            inst.sg:GoToState("idle")
        end
    end
end

local function fncommon()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    local physics = inst.entity:AddPhysics()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize( 2.5, 1.5 )
    inst.Transform:SetFourFaced()

	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "bunnyman.tex" )
	
	inst:AddTag("scarytoprey")
    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("hound")
	--所有敌对人形怪
	inst:AddTag("groundpoundimmune")
	inst:AddTag("toso_arm")
    inst:AddTag("toso_armboss")	
	
--两栖地 
    MakeAmphibiousCharacterPhysics(inst, 0, .5)
     
    anim:SetBank("hound")
    anim:SetBuild("hound")
    anim:PlayAnimation("idle")
    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.runspeed = TUNING.HOUND_SPEED

    inst:SetStateGraph("SGboss_hound")

    local brain = require "brains/houndbrain"
    inst:SetBrain(brain)

    inst:AddComponent("follower")
    
    inst:AddComponent("eater")
    inst.components.eater:SetCarnivore()
	inst.components.eater:SetCanEatHorrible()

    inst.components.eater.strongstomach = true -- can eat monster meat!

local player = GetPlayer()  
   
    inst:AddComponent("health")
	--生命护甲数值
	inst.components.health:SetMaxHealth(240000+player.level*TUNING.ARM_HEALTHUP)
    inst.components.health:SetAbsorptionAmount(TUNING.ARM_ARMOR)
    
    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED
  
    
    inst:AddComponent("combat")
	--攻速攻击力数值
	 inst.components.combat:SetDefaultDamage(400+player.level*20)
	inst.components.combat:SetAttackPeriod(TUNING.ARM_ATKSPEED*0.5)
	
    --inst.components.combat:SetRetargetFunction(3, retargetfn)
    --inst.components.combat:SetKeepTargetFunction(KeepTarget)
    inst.components.combat:SetRetargetFunction(0, assasin_hunt)
	inst.components.combat:SetKeepTargetFunction(Onhunt)
    inst.components.combat:SetHurtSound("dontstarve/creatures/hound/hurt")

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('hound')
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("sleeper")
    inst.components.sleeper:SetResistance(3)
    inst.components.sleeper.testperiod = GetRandomWithVariance(6, 2)
    inst.components.sleeper:SetSleepTest(ShouldSleep)
    inst.components.sleeper:SetWakeTest(ShouldWakeUp)
    inst:ListenForEvent("newcombattarget", OnNewTarget)

    inst:ListenForEvent( "dusktime", function() OnNight( inst ) end, GetWorld()) 
    inst:ListenForEvent( "nighttime", function() OnNight( inst ) end, GetWorld()) 
    inst.OnEntitySleep = OnEntitySleep

    inst.OnSave = OnSave
    inst.OnLoad = OnLoad

    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("onattackother", OnAttackOther)

    return inst
end

local function fndefault()
	local inst = fncommon(Sim)
    --领域
	inst:ListenForEvent("onhitother", houndatk)
	--成长护甲
	inst.task_always = inst:DoPeriodicTask(2, function()
	local player = GetPlayer()


	
if player.level > 10 and player.level <= 20 then
inst.components.health:SetAbsorptionAmount(0.91)
elseif player.level > 20 and player.level <= 30 then
inst.components.health:SetAbsorptionAmount(0.92)	
elseif player.level > 30 and player.level <= 40 then
inst.components.health:SetAbsorptionAmount(0.93)
elseif player.level > 40 and player.level <= 50 then
inst.components.health:SetAbsorptionAmount(0.94)
elseif player.level > 50 and player.level <= 60 then
inst.components.health:SetAbsorptionAmount(0.95)
elseif player.level > 60 and player.level <= 70 then
inst.components.health:SetAbsorptionAmount(0.96)
elseif player.level > 70 and player.level <= 80 then
inst.components.health:SetAbsorptionAmount(0.97)
elseif player.level > 80 and player.level <= 90 then
inst.components.health:SetAbsorptionAmount(0.98)
elseif player.level > 90 then
inst.components.health:SetAbsorptionAmount(0.99)
else
inst.components.health:SetAbsorptionAmount(TUNING.ARM_ARMOR)
end	
	
	end)
    ----两栖地 
    --MakeAmphibiousCharacterPhysics(inst, 1, .5)

	inst:DoPeriodicTask(900, function() 
	inst:DoTaskInTime(1.5, function()  SpawnPrefab("lightning").Transform:SetPosition(inst:GetPosition():Get())
	--inst:DoTaskInTime(0.25, function() SpawnPrefab("lightning").Transform:SetPosition(inst:GetPosition():Get()) 
	
	inst:DoTaskInTime(10, function() 
		local player = GetPlayer()
	--SpawnPrefab("lightning").Transform:SetPosition(inst:GetPosition():Get())	
	        local downvec = TheCamera:GetDownVec()
            local x,y,z = player.Transform:GetWorldPosition()	
            --local arm2 = inst
			local theta = math.random() * 2 * PI
			local radius = 25
            local offset = FindWalkableOffset(inst:GetPosition(), theta, radius, 12, true) or FindWaterOffset(inst:GetPosition(), theta, radius, 36, false) or FindWaterOffset(inst:GetPosition(), theta, radius, 36, false) or downvec:Normalize()
            inst.Transform:SetPosition(x + offset.x, y + offset.y, z + offset.z)
			inst.SoundEmitter:PlaySound("dontstarve/common/craftable/lightning_rod")
            SpawnPrefab("statue_transition_2").Transform:SetPosition(inst:GetPosition():Get()) 			
	
	end) 
	end) 
	end)
	
    MakeMediumFreezableCharacter(inst, "hound_body")
    MakeMediumBurnableCharacter(inst, "hound_body")
	return inst
end

local function fnfire(Sim)
	local inst = fncommon(Sim)
	inst.AnimState:SetBuild("hound_red")
	
    MakeMediumFreezableCharacter(inst, "hound_body")
    inst.components.freezable:SetResistance(4) --because fire

    inst.components.combat:SetDefaultDamage(10)
	inst.components.combat:SetAttackPeriod(0.8)
    inst.components.locomotor.runspeed = TUNING.FIREHOUND_SPEED
    inst.components.health:SetMaxHealth(TUNING.FIREHOUND_HEALTH)
    inst.components.lootdropper:SetChanceLootTable('hound_fire')


	inst:ListenForEvent("death", function(inst)
		inst.SoundEmitter:PlaySound("dontstarve/creatures/hound/firehound_explo", "explosion")
	end)
	
	return inst
end

local function fncold(Sim)
	local inst = fncommon(Sim)
	inst.AnimState:SetBuild("hound_ice")

    MakeMediumBurnableCharacter(inst, "hound_body")

    inst.components.combat:SetDefaultDamage(TUNING.ICEHOUND_DAMAGE)
    inst.components.combat:SetAttackPeriod(TUNING.ICEHOUND_ATTACK_PERIOD)
    inst.components.locomotor.runspeed = TUNING.ICEHOUND_SPEED
    inst.components.health:SetMaxHealth(TUNING.ICEHOUND_HEALTH)
    inst.components.lootdropper:SetChanceLootTable('hound_cold')
	
	inst:ListenForEvent("death", function(inst)
        if not inst.components.freezable then
            MakeMediumFreezableCharacter(inst, "hound_body")
        end
        inst.components.freezable:SpawnShatterFX()
        inst:RemoveComponent("freezable")
        local x,y,z = inst.Transform:GetWorldPosition()
        local ents = TheSim:FindEntities(x, y, z, 4, {"freezable"}, NO_TAGS) 
        for i,v in pairs(ents) do
            if v.components.freezable then
                v.components.freezable:AddColdness(2)
            end
        end

		inst.SoundEmitter:PlaySound("dontstarve/creatures/hound/icehound_explo", "explosion")
	end)

	return inst
end

local function fnfiredrop(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
    MakeInventoryPhysics(inst)

    MakeLargeBurnable(inst, 6+ math.random()*6)
    MakeLargePropagator(inst)
    inst.components.burnable:Ignite()
    inst.persists = false
    inst.components.burnable:SetOnExtinguishFn(function(inst) inst:Remove() end)
    return inst
end

local function fn_houndwind()
	local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()


    MakeInventoryPhysics(inst)
    RemovePhysicsColliders(inst)

    --inst.Transform:SetFourFaced()

    inst.AnimState:SetBank("tornado")
    inst.AnimState:SetBuild("tornado")
    inst.AnimState:PlayAnimation("tornado_pre")
    inst.AnimState:PushAnimation("tornado_loop")

    inst:AddTag("fx")



    inst:AddComponent("combat")
	
	----冰风暴
			inst.SoundEmitter:PlaySound("dontstarve_DLC001/common/tornado", "spinLoop")
	local period = .1
	inst._tick = 0
	inst.collide_task_table = {}
	inst:DoPeriodicTask(period, function()
	
		--伤害
		local pt = inst:GetPosition()	
		local r = 2.5
		local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, r, nil, {"FX", "NOCLICK", "DECOR", "INLIMBO", "shadow", "groundpoundimmune","toso_arm","shadowboss"})
		
		for k,v in pairs(ents) do
			if v and v.components.health and not v.components.health:IsDead() and v.components.combat then
				
				if not table.contains(inst.collide_task_table, v.GUID) then
									local player = GetPlayer()
					local demage = 200 + 10*player.level
					v.components.combat:GetAttacked(inst, demage)
					table.insert(inst.collide_task_table, v.GUID)
				end
				
			end
		end
		
		--发射
		local rot = inst.Transform:GetRotation()
		rot = rot + inst._tick * 360 / (1/period)
		while (rot > 360) do
			rot = rot - 360
		end
		inst._tick = inst._tick + 1
		
		--local fx = SpawnPrefab("zg_fx_wickerwind2")
		--if fx then
			--fx.Transform:SetPosition( pt:Get() )
			--fx.Transform:SetRotation( rot )
			--fx.Physics:SetMotorVelOverride(15, 0, 0)
		--end
		
	end)
	
	----冰风暴的生命周期
	inst.zg_lifetime = 1.5
	inst:DoTaskInTime(inst.zg_lifetime, function()
		if inst then
			inst.SoundEmitter:KillSound("spinLoop")
			inst:Remove()
		end
	end)

	return inst
end

return Prefab ("houndwind", fn_houndwind),
Prefab( "monsters/boss_hound", fndefault, assets, prefabs),
		Prefab( "monsters/boss_firehound", fnfire, assets, prefabs),
		Prefab( "monsters/boss_icehound", fncold, assets, prefabs),
		Prefab( "monsters/houndfire", fnfiredrop, assets, prefabs) 
