local assets=
{
	Asset("ANIM", "anim/chunhuideng.zip"),
	Asset("ANIM", "anim/swap_chunhuideng.zip"),
    Asset("SOUND", "sound/wilson.fsb"),
	Asset("IMAGE","images/inventoryimages/chunhuideng.tex"),
	Asset("ATLAS","images/inventoryimages/chunhuideng.xml")
}

local YESTAGS = {"burnable"}
local NOTAGS = {"FX", "DECOR", "INLIMBO", "flingomatic_freeze_immune"} -- "NOCLICK",

local function UnprotectPlant(inst, plant)
	if plant then
		local index = -1
		if plant and #inst.protected_plants > 0 then
			for k,v in ipairs(inst.protected_plants) do
				if v and v == plant then
					inst:RemoveEventCallback("picked", v.UnprotectPlant, v)
					v:RemoveTag("protected")
					v.components.pickable.protected = false
					v.makewitherabletask = v:DoTaskInTime(TUNING.WITHER_BUFFER_TIME, function(v) 
						v.components.pickable:MakeWitherable()
					end)
					index = k
					break
				end
			end
		end
		if index > 0 then
			table.remove(inst.protected_plants, index)
		end
	end
end



local function turnon(inst)
--显示范围
    local range = 15    
	local hitrangescale = math.sqrt(range * 300 / 1900)
	--local s1, s2, s3 = self.inst.Transform:GetScale()
	
	if inst and inst.rangedisplay == nil then
		inst.rangedisplay = SpawnPrefab("display_build_range")
	    inst.rangedisplay.Transform:SetScale(hitrangescale, hitrangescale, hitrangescale )
		inst.rangedisplay.entity:SetParent(inst.entity)
		inst.rangedisplay.AnimState:SetLightOverride(0)
		inst.rangedisplay.AnimState:SetSortOrder(1)
	end
    inst:DoTaskInTime(5, function() 
	if inst and inst.rangedisplay then
		inst.rangedisplay:Remove()
		inst.rangedisplay = nil
	end

	end)
	
	

    if not inst.components.fueled:IsEmpty() then
        if not inst.components.machine.ison then
            if inst.components.fueled then
                inst.components.fueled:StartConsuming()        
            end
            inst.Light:Enable(true)
            inst.AnimState:PlayAnimation("idle_on")

            inst.components.machine.ison = true

            inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_on")
            inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_LP", "loop")

        end
    end
end

local function turnoff(inst)
    if inst.components.fueled then
        inst.components.fueled:StopConsuming()        
    end

    inst.Light:Enable(false)
    inst.AnimState:PlayAnimation("idle_off")


    inst.components.machine.ison = false

    inst.SoundEmitter:KillSound("loop")
    inst.SoundEmitter:PlaySound("dontstarve/wilson/lantern_off")

end

local function OnLoad(inst, data)
    if inst.components.machine and inst.components.machine.ison then
        inst.AnimState:PlayAnimation("idle_on")
        turnon(inst)
    else
        inst.AnimState:PlayAnimation("idle_off")
        turnoff(inst)
    end
end

local function ondropped(inst)
    turnoff(inst)
    turnon(inst)
end

local function onpickup(inst)
	turnon(inst)
end

local function onputininventory(inst)
    turnoff(inst)
end
--
local function onequip(inst, owner)
    owner.AnimState:OverrideSymbol("swap_object", "swap_chunhuideng", "swap_light1")
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")

    turnon(inst)
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal")
end

local function nofuel(inst)
    local owner = inst.components.inventoryitem and inst.components.inventoryitem.owner
    if owner then
        owner:PushEvent("torchranout", {torch = inst})
    end

    turnoff(inst)
end

local function takefuel(inst)
    if inst.components.equippable and inst.components.equippable:IsEquipped() then
        turnon(inst)
    end
end

local function fuelupdate(inst)
    local fuelpercent = inst.components.fueled:GetPercent()
    inst.Light:SetIntensity(Lerp(0.4, 0.6, fuelpercent))
    inst.Light:SetRadius(Lerp(3, 5, fuelpercent))
    inst.Light:SetFalloff(.9)
end

local function HitPlants(inst, dist, noextinguish)
	
	local protector = inst.owner or inst
		
	dist = dist or 15

	local x,y,z = inst:GetPosition():Get()

	local ents = TheSim:FindEntities(x,y,z, dist, YESTAGS, NOTAGS)

	for k,v in pairs(ents) do
		if v then
			print("CHECK CHECK",v.prefab)
			if v.makewitherabletask then
				v.makewitherabletask:Cancel()
				v.makewitherabletask = nil
				table.insert(protector.protected_plants, v)
				v:AddTag("protected")
				if v.components.pickable then
					v.UnprotectPlant = function(v)
						protector:UnprotectPlant(v)
					end
					protector:ListenForEvent("picked", v.UnprotectPlant, v)
				end
				if v.components.crop then
					v.components.crop.protected = true
				elseif v.components.pickable then
					v.components.pickable.protected = true
				end
			elseif v.components.crop and v.components.crop.witherable then
				v.components.crop.protected = true
				table.insert(protector.protected_plants, v)
				v:AddTag("protected")
			elseif v.components.pickable and v.components.pickable.witherable then
				v.components.pickable.protected = true
				if v.components.pickable.withered or v.components.pickable.shouldwither then
					if v.components.pickable.cycles_left and v.components.pickable.cycles_left <= 0 then
		    			v.components.pickable:MakeBarren()
		    		else
		    			v.components.pickable:MakeEmpty()
		    		end
		    		v.components.pickable.withered = false
		    		v.components.pickable.shouldwither = false
		    		v:RemoveTag("withered")
				end
				table.insert(protector.protected_plants, v)
				v:AddTag("protected")
				v.UnprotectPlant = function(v)
					protector:UnprotectPlant(v)
				end
				protector:ListenForEvent("picked", v.UnprotectPlant, v)
			end

			if not noextinguish then
				print("testing",v.prefab)
				if v.components.burnable then
					print("testing 2",v.prefab)
					if v.components.burnable:IsBurning() then
						print("testing 3",v.prefab)
						v.components.burnable:Extinguish(true, TUNING.FIRESUPPRESSOR_EXTINGUISH_HEAT_PERCENT)
					elseif v.components.burnable:IsSmoldering() then
						print("testing 4",v.prefab)
						v.components.burnable:Extinguish(true)
					end
				end
				if v.components.freezable and not v:HasTag("toso_arm")then
					print("testing 5",v.prefab)
					v.components.freezable:AddColdness(2) 
				end
				if v.components.temperature then
					print("testing 6",v.prefab)
					local temp = v.components.temperature:GetCurrent()
	        		v.components.temperature:SetTemperature(temp - TUNING.FIRE_SUPPRESSOR_TEMP_REDUCTION)
				end
			end
		end
	end
end

local function OnLoadPostPass(inst, data)
	if not inst.components.fueled:IsEmpty() then
		HitPlants(inst, TUNING.FIRE_DETECTOR_RANGE, true)
	end
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()        
    MakeInventoryPhysics(inst)
    
    inst.AnimState:SetBank("light1")
    inst.AnimState:SetBuild("chunhuideng")
    inst.AnimState:PlayAnimation("idle_off")
	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon( "chunhuideng.tex" )
	
    inst:AddTag("light")

    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")

    inst.components.inventoryitem:SetOnDroppedFn(ondropped)
    --inst.components.inventoryitem:SetOnPickupFn(makesmalllight)
    --inst.components.inventoryitem:SetOnActiveItemFn(makesmalllight)
    inst.components.inventoryitem:SetOnPutInInventoryFn(onputininventory)
	inst.components.inventoryitem.imagename = "chunhuideng"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/chunhuideng.xml"

    inst:AddComponent("equippable")

    inst:AddComponent("fueled")

    inst:AddComponent("machine")
    inst.components.machine.turnonfn = turnon
    inst.components.machine.turnofffn = turnoff
    inst.components.machine.cooldowntime = 0
    inst.components.machine.caninteractfn = function() return not inst.components.fueled:IsEmpty() and (inst.components.inventoryitem.owner == nil or inst.components.equippable.isequipped) end


    inst.components.fueled.fueltype = "CAVE"
    inst.components.fueled:InitializeFuelLevel(TUNING.LANTERN_LIGHTTIME*5)
    inst.components.fueled:SetDepletedFn(nofuel)
    inst.components.fueled:SetUpdateFn(fuelupdate)
    inst.components.fueled.ontakefuelfn = takefuel
    inst.components.fueled.accepting = true

    inst.entity:AddLight()
    inst.Light:SetColour(180/255, 195/255, 150/255)

    fuelupdate(inst)

    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip ) 

	inst:AddComponent("firedetector")
	--inst.components.firedetector:SetOnFindFireFn(OnFindFire)
	inst.protected_plants = {}
	inst.UnprotectPlant = UnprotectPlant

    inst.OnLoad = OnLoad
    inst.OnLoadPostPass = OnLoadPostPass
	
	inst.task_always = inst:DoPeriodicTask(4, function()
	
    if inst:IsValid() and not inst.components.fueled:IsEmpty() and inst.components.machine.ison then
	HitPlants(inst)
	end	
	
	
	end)

    return inst
end


return Prefab( "common/inventory/chunhuideng", fn, assets)
