require "prefabutil"

local notags = {'NOBLOCK', 'player', 'FX'}
local function test_ground(inst, pt)
	local tiletype = GetGroundTypeAtPosition(pt)
	local ground_OK = tiletype ~= GROUND.ROCKY and tiletype ~= GROUND.ROAD and tiletype ~= GROUND.IMPASSABLE and
						tiletype ~= GROUND.UNDERROCK and tiletype ~= GROUND.WOODFLOOR and 
						tiletype ~= GROUND.CARPET and tiletype ~= GROUND.CHECKER and tiletype < GROUND.UNDERGROUND
	
	if ground_OK then
	    local ents = TheSim:FindEntities(pt.x,pt.y,pt.z, 4, nil, notags)
		local min_spacing = inst.components.deployable.min_spacing or 2

	    for k, v in pairs(ents) do
			if v ~= inst and v.entity:IsValid() and v.entity:IsVisible() and not v.components.placer and v.parent == nil then
				if distsq( Vector3(v.Transform:GetWorldPosition()), pt) < min_spacing*min_spacing then
					return false
				end
			end
		end
		
		return true

	end
	return false
	
end


local function make_plantable(data)
	local assets = 
		{
	Asset("ANIM", "anim/bamboobush.zip"),	 
	Asset("ATLAS", "images/inventoryimages/bamboobush.xml"),
	Asset("IMAGE", "images/inventoryimages/bamboobush.tex"),
	Asset("SOUND", "sound/common.fsb"),
		}
	if data.build then
		table.insert(assets, Asset("ANIM", "anim/"..data.build..".zip"))
	end

	local function ondeploy(inst, pt)
		local tree = SpawnPrefab("bamboobush") 
		if tree then 
			tree.Transform:SetPosition(pt.x, pt.y, pt.z) 
			inst.components.stackable:Get():Remove()
			tree.components.pickable:OnTransplant()
		end 
	end
	
	local function fn(Sim)
		local inst = CreateEntity()
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()
		MakeInventoryPhysics(inst)
	    
		inst.AnimState:SetBank("berrybush")
		inst.AnimState:SetBuild("bamboobush")
		inst.AnimState:PlayAnimation("dropped")

		inst:AddComponent("stackable")
		inst.components.stackable.maxsize = 20
		inst:AddTag("yiyuonly")
		inst:AddComponent("inspectable")

		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.imagename = "bamboobush"
		inst.components.inventoryitem.atlasname = "images/inventoryimages/bamboobush.xml"
			    
		inst:AddComponent("fuel")
		inst.components.fuel.fuelvalue = TUNING.LARGE_FUEL

        MakeMediumBurnable(inst, TUNING.LARGE_BURNTIME)
		MakeSmallPropagator(inst)
		inst.components.burnable:MakeDragonflyBait(3)
		
	    inst:AddComponent("deployable")
	    --inst.components.deployable.test = function() return true end
	    inst.components.deployable.ondeploy = ondeploy
	    inst.components.deployable.test = test_ground
	    inst.components.deployable.min_spacing = data.minspace or 2

		if TUNING.SDLC == 2 or TUNING.SDLC == 3 then
		MakeInventoryFloatable(inst, "dropped", "dropped")
		end
		---------------------  
		return inst      
	end

	return Prefab( "common/objects/dug_bamboobush", fn, assets)
end

local plantables = 
{
	{name="bamboobush", bank = "berrybush", anim="idle_dead", minspace=2},
}

local prefabs= {}
for k,v in pairs(plantables) do
	table.insert(prefabs, make_plantable(v))
	table.insert(prefabs, MakePlacer( "common/dug_"..v.name.."_placer", v.bank or v.name, v.build or v.name, v.anim or "idle" ))
end

return unpack(prefabs) 