local assets =
{
	Asset("ANIM", "anim/inventoryanim/jiandai.zip"),
	Asset("ANIM", "anim/inventoryanim/hat_hulu.zip"),
    Asset("ATLAS", "images/inventoryimages/hulu.xml"),
    Asset("IMAGE", "images/inventoryimages/hulu.tex"),
}

---ORANGE
local function SpawnEffect(inst)
    local pt = inst:GetPosition()
    local fx = SpawnPrefab("small_puff")
    fx.Transform:SetPosition(pt.x, pt.y, pt.z)
    fx.Transform:SetScale(0.5,0.5,0.5)
end

local function getitem(player, amulet, item)
    --Amulet will only ever pick up items one at a time. Even from stacks.
    SpawnEffect(item)
    --amulet.components.finiteuses:Use(1)
    
    if item.components.stackable then
        item = item.components.stackable:Get()
    end
    
    if item.components.trap and item.components.trap:IsSprung() then
        item.components.trap:Harvest(player)
        return
    end
    
    player.components.inventory:GiveItem(item)
end

local function pickup(inst, owner)
    local pt = owner:GetPosition()
    local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, TUNING.ORANGEAMULET_RANGE)

    for k,v in pairs(ents) do
        if v.components.inventoryitem and v.components.inventoryitem.canbepickedup and v.components.inventoryitem.cangoincontainer and not
            v.components.inventoryitem:IsHeld() and v.prefab ~= "toso_packim" and v.prefab ~= "toso_xiang" then

            if not owner.components.inventory:IsFull() then
                --Your inventory isn't full, you can pick something up.
                getitem(owner, inst, v)
                return

            elseif v.components.stackable then
                --Your inventory is full, but the item you're trying to pick up stacks. Check for an exsisting stack.
                --An acceptable stack should: Be of the same item type, not be full already and not be in the "active item" slot of inventory.
                local stack = owner.components.inventory:FindItem(function(item) return (item.prefab == v.prefab and not item.components.stackable:IsFull()
                    and item ~= owner.components.inventory.activeitem) end)
                if stack then
                    getitem(owner, inst, v)
                    return
                end
            end
        end
    end
    
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_hat", "hat_hulu", "swap_hat")

    owner.AnimState:Show("HAT")
    owner.AnimState:Show("HAT_HAIR")
    owner.AnimState:Hide("HAIR_NOHAT")
    owner.AnimState:Hide("HAIR")
    --owner.AnimState:OverrideSymbol("swap_body", "torso_amulets", "orangeamulet")
    inst.task = inst:DoPeriodicTask(0.2, function() pickup(inst, owner) end)
end

local function onunequip(inst, owner) 
    owner.AnimState:Hide("HAT")
    owner.AnimState:Hide("HAT_HAIR")
    owner.AnimState:Show("HAIR_NOHAT")
    owner.AnimState:Show("HAIR")
    --owner.AnimState:ClearOverrideSymbol("swap_body")
    if inst.task then inst.task:Cancel() inst.task = nil end
end

local function onopen(inst)
	inst.AnimState:PlayAnimation("idle", false)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
end

local function onclose(inst)
	inst.AnimState:PlayAnimation("idle", false)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
end

local slotpos = {}
		for y = 4, 0, -1 do
			for x = 0, 15 do
				table.insert(slotpos, Vector3(80*x-346*2+91, 80*y-100*2+42,0))
			end
		end


local function itemtest(inst, item, slot)
	return not item:HasTag("notbackpack")
end


local function fn()
	local inst = CreateEntity()
	inst:AddTag("fridge")
    
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    --inst.entity:AddMiniMapEntity()

	--inst.backpack_level = 0
	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon( "hulu.tex" )
	
    inst.AnimState:SetBank("jiandai")
    inst.AnimState:SetBuild("jiandai")
    inst.AnimState:PlayAnimation("idle")

    --inst.MiniMapEntity:SetIcon("backpack.png")
    
    --inst.foleysound = "dontstarve/movement/foley/backpack"
    
    inst:AddTag("notbackpack")

	
	
    inst:AddComponent("inspectable")
    
    inst:AddComponent("inventoryitem")
	--inst.components.inventoryitem.keepondeath = true
	inst.components.inventoryitem.imagename = "hulu"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/hulu.xml"
	inst.components.inventoryitem.cangoincontainer = true

	inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
    inst.components.equippable.equipslot = EQUIPSLOTS.NECK or EQUIPSLOTS.HEAD

    inst:AddComponent("container")
    inst.components.container.itemtestfn = itemtest
    inst.components.container:SetNumSlots(#slotpos)
    inst.components.container.onopenfn = onopen
	inst.components.container.onclosefn = onclose
    inst.components.container.widgetslotpos = slotpos
    inst.components.container.widgetanimbank = "ui_chest_5x16"
    inst.components.container.widgetanimbuild = "ui_chest_5x16"
    inst.components.container.widgetpos = Vector3(0,190,0)
    inst.components.container.side_align_tip = 160
    --inst.components.container.side_widget = true
    --inst.components.container.type = "pack"
	--inst.components.container.widgetbuttoninfo = widgetbuttoninfo


    return inst
end

return Prefab("common/inventory/hulu", fn, assets)