local PopupDialogScreen = require "screens/popupdialog"
local ImageButton = require "widgets/imagebutton"
local Text = require "widgets/text"

local assets=
{
    Asset("ATLAS", "images/inventoryimages/kid_eyebone.xml"),
    Asset("ANIM", "anim/kid_eyebone.zip"),
    Asset("ANIM", "anim/swap_eyebone.zip"),
}

--装备
local function onequip(inst, owner)

    owner.AnimState:OverrideSymbol("swap_object", "swap_eyebone", "swap_futou")
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")

	
end

local function onunequip(inst, owner)

    owner.AnimState:Hide("ARM_carry")
    owner.AnimState:Show("ARM_normal")

end


local function RebuildTile(inst)
    if inst.components.inventoryitem:IsHeld() then
        local owner = inst.components.inventoryitem.owner
        inst.components.inventoryitem:RemoveFromOwner(true)
        if owner.components.container then
            owner.components.container:GiveItem(inst)
        elseif owner.components.inventory then
            owner.components.inventory:GiveItem(inst)
        end
    end
end

local function MorphUpBell(inst)
    RebuildTile(inst)
    local qiuqiu = TheSim:FindFirstEntityWithTag("qiuqiu")
    if qiuqiu then
        if qiuqiu.components.follower.leader ~= GetPlayer() then
            qiuqiu.components.follower:SetLeader(GetPlayer())
        end
	end
end

--------------------------------------------------------------
local function GetSpawnPoint(pt)
    local theta = math.random() * 2 * PI
    local radius = 30
	local offset = FindWalkableOffset(pt, theta, radius, 12, true)
	if offset then
		return pt+offset
	end
end

local function Spawnqiuqiu(inst)
    local pt = Vector3(inst.Transform:GetWorldPosition())
    local spawn_pt = GetSpawnPoint(pt)
    if spawn_pt then
        local qiuqiu = SpawnPrefab("pod")
        if qiuqiu then
			if inst.save_level and inst.save_level == 2 then
				qiuqiu.qiuqiu_level = 2
				if qiuqiu.upgrade then qiuqiu.upgrade(qiuqiu) end
				print("球球重生2")
			end
			if inst.save_level and inst.save_level == 3 then
				qiuqiu.qiuqiu_level = 3
				if qiuqiu.upgrade then qiuqiu.upgrade(qiuqiu) end
				print("球球重生3")
			end
            qiuqiu.Physics:Teleport(spawn_pt:Get())
			qiuqiu:FacePoint(pt.x, pt.y, pt.z)
			return qiuqiu
		end
	end
end

local function StopRespawn(inst)
    if inst.respawntask then
        inst.respawntask:Cancel()
        inst.respawntask = nil
        inst.respawntime = nil
    end
end

local function Rebindqiuqiu(inst, qiuqiu)
    local qiuqiu = TheSim:FindFirstEntityWithTag("qiuqiu")
    if qiuqiu then
        inst.AnimState:PlayAnimation("idle", true)
        inst:ListenForEvent("death", function() inst:OnDeath() end, qiuqiu)
        if qiuqiu.components.follower.leader ~= GetPlayer() then
            qiuqiu.components.follower:SetLeader(GetPlayer())
        end
        return true
    end
end

local function Respawnqiuqiu(inst)
    StopRespawn(inst)
    local qiuqiu = TheSim:FindFirstEntityWithTag("qiuqiu")
    if not qiuqiu then qiuqiu = Spawnqiuqiu(inst) end
    Rebindqiuqiu(inst, qiuqiu)
end

local function StartRespawn(inst, time)
    StopRespawn(inst)
    local respawntime = time or 0
    if respawntime then
        inst.respawntask = inst:DoTaskInTime(respawntime, function() Respawnqiuqiu(inst) end)
        inst.respawntime = GetTime() + respawntime
        inst.AnimState:PlayAnimation("idle", true)
    end
end

local function OnDeath(inst)
	StartRespawn(inst, 5)
end

local function Fixqiuqiu(inst)
	inst.fixtask = nil
	if GetWorld():HasTag("volcano") then
	inst:AddTag("chester_eyebone")
	elseif GetWorld():HasTag("shipwrecked") then
	inst:AddTag("chester_eyebone")
	else
	inst:RemoveTag("chester_eyebone")
	end

	if not Rebindqiuqiu(inst) then
        inst.AnimState:PlayAnimation("idle", true)
		if inst.components.inventoryitem.owner then
			local time_remaining = 0
			local time = GetTime()
			if inst.respawntime and inst.respawntime > time then
				time_remaining = inst.respawntime - time		
			end
			StartRespawn(inst, 5)
		end
	end
end

local function OnDropped(inst)
    inst.AnimState:PlayAnimation("idle", true)
	local qiuqiu = TheSim:FindFirstEntityWithTag("qiuqiu")
	if qiuqiu and qiuqiu.components.follower.leader ~= inst then
		qiuqiu.components.follower:SetLeader(inst)
		qiuqiu.components.combat:SetTarget(nil)
		qiuqiu.components.combat.canattack = false
		qiuqiu:AddTag("mode1")
		qiuqiu:RemoveTag("mode2")
		qiuqiu:RemoveTag("mode3")
	end
end

local function OnPutInInventory(inst)
	if not inst.fixtask then
		inst.fixtask = inst:DoTaskInTime(1, function() Fixqiuqiu(inst) end)	
	end
end

local function OnSave(inst, data)
    local time = GetTime()
    if inst.respawntime and inst.respawntime > time then
        data.respawntimeremaining = inst.respawntime - time
    end
	if inst.save_level then data.save_level = inst.save_level end
end

local function OnLoad(inst, data)
    if data then inst:MorphUpBell() end
    if data and data.respawntimeremaining then
		inst.respawntime = data.respawntimeremaining + GetTime()
	end
	if data.save_level then inst.save_level = data.save_level end
end

local function yiyuclosescreen()
	TheFrontEnd:PopScreen()
	SetPause(false)
end

local function onuseapple(inst)
	local player = GetPlayer()
	local pigkingnum = c_countprefabs("pigking", true) or 0
	local statueglommernum = c_countprefabs("statueglommer", true) or 0
	local tallbirdnestnum = c_countprefabs("tallbirdnest", true) or 0
	local walrus_campnum = c_countprefabs("walrus_camp", true) or 0
	local houndmoundnum = c_countprefabs("houndmound", true) or 0
	local rabbithousenum = c_countprefabs("rabbithouse", true) or 0
	local mandrakenum = c_countprefabs("mandrake", true) or 0
	local bishopnum = c_countprefabs("bishop", true) or 0
	local canenum = c_countprefabs("cane", true) or 0
	local livingtreenum = c_countprefabs("livingtree", true) or 0
	local ancient_altarnum = c_countprefabs("ancient_altar", true) or 0
	local mermhouse_fishernum = c_countprefabs("mermhouse_fisher", true) or 0
	local livingjungletreenum = c_countprefabs("livingjungletree", true) or 0
	local volcanostaffnum = c_countprefabs("volcanostaff", true) or 0
	
	local emerald_flowernum = c_countprefabs("emerald_flower", true) or 0	
--	player.components.talker.colour = Vector3(1, 0.75, 1, 1)
--	player.components.talker.fontsize = 24
--	print(inst.save_level.."MY_QQ_LEVEL")

if not IsPaused() then
	local yiyu_sp_doalog = PopupDialogScreen(
		"YIYU_APPLE",
		"\n"..TUNING.THERE..TUNING.THERE1.."["..pigkingnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE2.."["..statueglommernum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE3.."["..tallbirdnestnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE4.."["..walrus_campnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE5.."["..houndmoundnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE6.."["..rabbithousenum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE7.."["..mandrakenum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE8.."["..bishopnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE9.."["..canenum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE10.."["..livingtreenum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE11.."["..ancient_altarnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE12.."["..mermhouse_fishernum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE13.."["..livingjungletreenum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE14.."["..volcanostaffnum.."]"..
		"\n"..TUNING.THERE..TUNING.THERE16.."["..emerald_flowernum.."]",
	{ { text = TUNING.YIYU_YINCANG15, cb = yiyuclosescreen } })
	yiyu_sp_doalog:SetTitleTextSize(36)
	yiyu_sp_doalog:SetButtonTextSize(30)
	yiyu_sp_doalog.title:SetPosition(0, 200, 0)
	yiyu_sp_doalog.text:SetSize(25)
	yiyu_sp_doalog.text:SetRegionSize(900, 600)
	yiyu_sp_doalog.text:SetPosition(150, 10, 0)
	yiyu_sp_doalog.text:SetHAlign(ANCHOR_LEFT)
	yiyu_sp_doalog.bg:SetScale(1.5, 1.9, 1.2)
	yiyu_sp_doalog.menu:SetPosition(0, -200, 0)
	TheFrontEnd:PushScreen(yiyu_sp_doalog)
	SetPause(true,"pause")
end
end

local function fn(Sim)
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
	
    inst.entity:AddMiniMapEntity()
    --inst.MiniMapEntity:SetIcon("yiyu_apple.tex")
	inst.entity:AddSoundEmitter()

    inst:AddTag("kid_eyebone")
    inst:AddTag("irreplaceable")
    inst:AddTag("follower_leash")
--	inst:AddTag("nonpotatable")
	inst.save_level = 1

	MakeInventoryPhysics(inst)
    inst.entity:AddDynamicShadow()
    inst.DynamicShadow:SetSize( 1, .5 )
    
    inst.AnimState:SetBank("eyebone")
    inst.AnimState:SetBuild("kid_eyebone")
    inst.AnimState:PlayAnimation("idle")

	inst:AddComponent("characterspecific")
    inst.components.characterspecific:SetOwner("ssword")

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem:SetOnPutInInventoryFn(OnPutInInventory)
	inst.components.inventoryitem.atlasname = "images/inventoryimages/kid_eyebone.xml"

	--if TUNING.YIYUDLC == 2 or TUNING.YIYUDLC == 3 then
	MakeInventoryFloatable(inst, "idle", "idle")
	--end

    inst:AddComponent("inspectable")
    --inst.components.inspectable.getstatus = onuseapple
	inst.components.inspectable:RecordViews()

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.WALKING_STICK_DAMAGE)
	
    inst:AddComponent("equippable")
    
    inst.components.equippable:SetOnEquip( onequip )
    inst.components.equippable:SetOnUnequip( onunequip )
    inst.components.equippable.walkspeedmult = TUNING.WALKING_STICK_SPEED_MULT

    inst:AddComponent("talker")
    inst.components.talker.fontsize = 24
	inst.components.talker.offset = Vector3(0,-800,0)
    inst.components.talker.colour = Vector3(0.75, 0.85, 1, 1)

    inst:AddComponent("leader")
--	inst.MorphNormalBell = MorphNormalBell
	inst.MorphUpBell = MorphUpBell

    inst.OnLoad = OnLoad
    inst.OnSave = OnSave
    inst.OnDeath = OnDeath

	inst:ListenForEvent("entity_death", function(world, data) OnDropped(inst, data.inst) end, GetWorld())
	inst.fixtask = inst:DoTaskInTime(1, function() Fixqiuqiu(inst) end)

    return inst
end

return Prefab( "common/inventory/kid_eyebone", fn, assets)