require "prefabutil"

local assets=
{
    Asset("ANIM", "anim/python_fountain.zip"),  
	--Asset("ANIM", "anim/firefighter_meter.zip"),
}

local prefabs =
{
	"collapse_small",
	"water_spray",
	"sprinkler_placer",
}

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
	inst:Remove()
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("off")
	--inst.AnimState:PushAnimation("idle_full", true)
end

local function checkshuidao(inst)
	local x,y,z = inst.Transform:GetWorldPosition()
	local ents = TheSim:FindEntities(x, 0, z, 16, {"onikiri_rice_plant"})
	for i, v in ipairs(ents) do
		v:DoTaskInTime(1,function()
			v:CheckSelf()
		end)
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("flow_pre")
	inst.AnimState:PushAnimation("flow_loop", true)
	inst.SoundEmitter:PlaySound("dontstarve_DLC001/common/craftable/firesupressor")
	inst:DoTaskInTime(0,function()
		--checkshuidao(inst)
	end)
end

local function UpdateSpray(inst)
    local x, y, z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, y, z, RANGE)

	if not inst.moisture_targets then
		inst.moisture_targets = {}
	end
	inst.moisture_targets_old = {}
	for GUID,v in pairs(inst.moisture_targets) do
    	inst.moisture_targets_old[GUID] = v
	end
    inst.moisture_targets = {} 

    for k,v in pairs(ents) do
    	inst.moisture_targets[v.GUID] = v
		if v.components.moisture then
			if not v.components.moisture.moisture_sources then
				v.components.moisture.moisture_sources = {}
			end
			v.components.moisture.moisture_sources[inst.GUID] = inst.moisturizing			
		end

		if v.components.moisturelistener and not (v.components.inventoryitem and v.components.inventoryitem.owner) then

			v.components.moisturelistener:AddMoisture(TUNING.MOISTURE_SPRINKLER_PERCENT_INCREASE_PER_SPRAY)
		end


		if v.components.crop and v.components.crop.task then

			v.components.crop.growthpercent = v.components.crop.growthpercent + (0.001)
		end

		if v.components.burnable and not (v.components.inventoryitem and v.components.inventoryitem.owner) then
			v.components.burnable:Extinguish()
		end

		if v.growwithsprinkler then
			v:AddTag("sprinkled")
			v.testForGrowth(v)
		end		
	end

	for GUID,v in pairs(inst.moisture_targets_old)do
		local still_affected = false
		for iGUID, i in pairs(inst.moisture_targets)do
			if GUID == iGUID then
				still_affected = true
				break
			end
		end
		if not still_affected then	
			if v.components.moisture then
				v.components.moisture.moisture_sources[inst.GUID] = nil
			end
			if v.growwithsprinkler then
				v:RemoveTag("sprinkled")
				v.testForGrowth(v)
			end			
		end
	end
end

local function OnRemove(inst)
	--checkshuidao(inst)
	
	if inst.waterSpray then
		inst.waterSpray:Remove()
		inst.waterSpray = nil
	end

	if inst.droptask then
		inst.droptask:Cancel()
		inst.droptask = nil
	end

	if inst.spraytask then
		inst.spraytask:Cancel()
		inst.spraytask = nil
	end

	if inst.moisture_targets then
		for GUID, i in pairs(inst.moisture_targets)do
			--print("TURN OFF",i.prefab)
			if i.components.moisture then
				i.components.moisture.moisture_sources[inst.GUID] = nil
			end
			if i.growwithsprinkler then
				i:RemoveTag("sprinkled")
				i.testForGrowth(i)
			end
		end
	end
end

local RANGE = 8 
local function spawndrop(inst)
	local drop = SpawnPrefab("raindrop")
	local pt = Vector3(inst.Transform:GetWorldPosition())
	local angle = math.random()*2*PI
	local dist = math.random()*RANGE
	local offset = Vector3(dist * math.cos( angle ), 0, -dist * math.sin( angle ))
	drop.Transform:SetPosition(pt.x+offset.x,0,pt.z+offset.z)
end

local function onwater(inst)
	if not inst.waterSpray then
		inst.waterSpray = inst:SpawnChild("water_spray")
		inst.waterSpray.Transform:SetPosition(0, 6, 0)
	end
end
--获取范围
local function getstatus(inst)
    local range = 8    
	local hitrangescale = math.sqrt(range * 300 / 1900)
	--local s1, s2, s3 = self.inst.Transform:GetScale()
	
	
	if inst and inst.rangedisplay == nil then
		inst.rangedisplay = SpawnPrefab("display_build_range")
		inst.rangedisplay.Transform:SetScale(hitrangescale, hitrangescale, hitrangescale )
		inst.rangedisplay.entity:SetParent(inst.entity)
		inst.rangedisplay.AnimState:SetLightOverride(0)
		inst.rangedisplay.AnimState:SetSortOrder(1)
	end
    inst:DoTaskInTime(5, function() 
	if inst and inst.rangedisplay then
		inst.rangedisplay:Remove()
		inst.rangedisplay = nil
	end

	end)
end

local function fn(Sim)
	local inst = CreateEntity()
	
    inst:AddTag("structure")
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    
	--local minimap = inst.entity:AddMiniMapEntity()
	--minimap:SetIcon( "onikiri_shuichi.tex" )
	
	MakeObstaclePhysics(inst, 1)
	
    inst.AnimState:SetBank("fountain")
    inst.AnimState:SetBuild("python_fountain")
    inst.AnimState:PlayAnimation("flow_loop",true)
	
	inst:AddTag("structure")
	inst:AddTag("onikiri_build_shuichi")
	
	--inst.AnimState:OverrideSymbol("swap_meter", "firefighter_meter", 15)
	
    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = getstatus
--[[
	local old = inst.components.inspectable.GetDescription
    inst.components.inspectable.GetDescription = function(self,viewer)
		if not (inst.fx and inst.fx:IsValid()) then
			inst.fx = inst:SpawnChild("onikiri_build_shuichi_fx")
		end
        inst.time = 5
		return old(self,viewer)
	end
	inst:ListenForEvent("newplacer",function() 
        if inst and inst:IsAsleep() then return end
        if not (inst.fx and inst.fx:IsValid()) then
            inst.fx = inst:SpawnChild("onikiri_build_shuichi_fx")
        end
        inst.time = 5
    end,GetWorld())
    inst:DoPeriodicTask(1,function()
        inst.time = (inst.time or 0 )- 1 
        if inst.time < 1 and inst.fx then
               inst:RemoveChild(inst.fx)
        end
    end)
]]
    inst:AddComponent("lootdropper")
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(5)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit) 
	
    inst:ListenForEvent( "onbuilt", onbuilt)
	inst:ListenForEvent( "onremove", OnRemove)
	
	inst.moisturizing = 2
	inst.UpdateSpray = UpdateSpray
	inst.waterSpray = nil
	
	inst:DoTaskInTime(0,onwater)
	inst.droptask = inst:DoPeriodicTask(0.2,function() spawndrop(inst) spawndrop(inst) end)
	inst.spraytask = inst:DoPeriodicTask(0.2,function()
		inst.UpdateSpray(inst)
	end)	
	
    return inst
end

local function fxfxfx()
	local inst = CreateEntity()
	
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	
    inst.AnimState:SetBank("onikiri_shuichi_fx")
    inst.AnimState:SetBuild("onikiri_shuichi_fx")
    inst.AnimState:PlayAnimation("idle",true)
    inst.AnimState:SetFinalOffset(-1)

    inst.AnimState:SetOrientation( ANIM_ORIENTATION.OnGround )
    inst.AnimState:SetLayer( LAYER_BACKGROUND )
    inst.AnimState:SetSortOrder( 3 )
	inst.Transform:SetScale(1.44, 1.44, 1.44)
    inst:DoTaskInTime(5,inst.Remove)
    inst.persists = false
    inst:AddTag("fx")
	return inst
end

local function test_ground(inst, pt)
	local tiletype = GetGroundTypeAtPosition(pt)
	local ground_OK = tiletype ~= GROUND.INTERIOR and tiletype ~= GROUND.GASJUNGLE and 
                      tiletype ~= GROUND.COBBLEROAD and tiletype ~= GROUND.FOUNDATION and 
                      tiletype ~= GROUND.ROCKY and tiletype ~= GROUND.ROAD and tiletype ~= GROUND.IMPASSABLE and
						tiletype ~= GROUND.UNDERROCK and tiletype ~= GROUND.WOODFLOOR and 
						tiletype ~= GROUND.CARPET and tiletype ~= GROUND.CHECKER and tiletype < GROUND.UNDERGROUND
	
	if ground_OK then
		return true
	end
	return false
end

local function placeTestFn(inst, pt)
    return true -- test_ground(inst, pt)
end
return Prefab( "ss_pool", fn, assets, prefabs),
	--Prefab( "onikiri_build_shuichi_fx", fxfxfx),
		MakePlacer("ss_pool_placer", "pugalisk_fountain", "pugalisk_fountain", "flow_loop",true, nil, nil, 1.44,nil, nil, nil, nil, nil, placeTestFn) 