local OnikiriUtil = {}
--判定鬼切的敌人inst能否设置target为目标
OnikiriUtil.HostileCanAttack = function(inst,target)
	--不攻击不合法的目标
	if not (target.components.combat and target.components.health and inst.components.combat:IsValidTarget(target)) then 
		return false
	end 

	--不攻击同为onikiri_hostile的目标
	if target:HasTag("onikiri_hostile") then 
		return false
	end 

	--攻击同类者必须受到攻击
	if target.components.combat.target and target.components.combat.target:HasTag("onikiri_hostile") then 
		return true
	end 

	--正常情况下，攻击一切友善生物(包括玩家)
	if target:HasTag("player") or not (target:HasTag("hostile") and target:HasTag("monster")) then 
		return true
	end 
end

OnikiriUtil.AllyCanAttack = function(inst,target)
	if not (target.components.combat and target.components.health and inst.components.combat:IsValidTarget(target)) then 
		return false
	end 

	if target:HasTag("companion") or target:HasTag("player") then 
		return false
	end 

	-- local targets_target = target.components.combat.target

	if target:HasTag("hostile") or target:HasTag("monster") or target:HasTag("onikiri_hostile") then 
		return true
	end 

	return true
end



local function HostileRetarget(inst,radius)
	return FindEntity(
        inst,
        radius or 20,
        function(guy)
            return OnikiriUtil.HostileCanAttack(inst,guy)
        end,
        { "_combat"},
        {"FX", "NOCLICK","INLIMBO", "monster","hostile","onikiri_hostile" }
    )
end

local function HostileKeepTarget(inst,target)
	return inst.components.combat:CanTarget(target)
end

local function HostileOnAttacked(inst,data,radius)
	radius = radius or 20 
	local attacker = data.attacker
	inst.components.combat:SuggestTarget(attacker)
	
	inst.components.combat:ShareTarget(attacker, radius, function(dude)
        return  (dude:HasTag("onikiri_hostile"))
            and (dude.components.health and not dude.components.health:IsDead()) 
    end,10)
end

OnikiriUtil.MakeOnikiriHostile = function(inst,data)
	data = data or {}

	inst:AddTag("onikiri_hostile")

	inst.components.combat:SetRetargetFunction(data.retargettime or 2, HostileRetarget)
	inst.components.combat:SetKeepTargetFunction(HostileKeepTarget)

	--------------防止范围攻击误伤中立monster生物
	local oldDoAreaAttack = inst.components.combat.DoAreaAttack
	inst.components.combat.DoAreaAttack = function(self,target, range, weapon, validfn, stimuli, excludetags,...)
		local old_validfn = validfn
		validfn = function(enemy,me)
			return (old_validfn == nil or old_validfn(enemy,me)) and OnikiriUtil.HostileCanAttack(self.inst,target)
		end
		return oldDoAreaAttack(self,target, range, weapon, validfn, stimuli, excludetags,...)
	end

	inst:ListenForEvent("attacked", HostileOnAttacked)

	if data.extrafn then
		data.extrafn(inst)
	end
end

OnikiriUtil.AddEpicBGM = function(inst,data)

		data = data or {}
		local bgmname = data.bgmname
		local startfn = data.startfn

		local function PushMusic(inst, level)
	    	if not (startfn == nil or startfn(inst)) then 
				return 
			end 
		    if GetPlayer() == nil then
		        inst._playingmusic = false
		    elseif GetPlayer():IsNear(inst, inst._playingmusic and 30 or 20) then
		        inst._playingmusic = true
		        GetPlayer():PushEvent("triggeredevent", { name = bgmname, level = level })
		    elseif inst._playingmusic and not GetPlayer():IsNear(inst, 40) then
		        inst._playingmusic = false
		    end
		end

		local function OnMusicDirty(inst)

		    if inst._musictask ~= nil then
		        inst._musictask:Cancel()
		    end
		    PushMusic(inst, inst._music)
		    inst._musictask = inst:DoPeriodicTask(1, PushMusic, nil, inst._music)
		    PushMusic(inst, inst._music)
		end

		local function SetMusicLevel(inst, level)
		    if inst._music ~= level then
		        inst._music = level
		        OnMusicDirty(inst)
		    end
		end

		inst:AddTag("noepicmusic")
		inst._music = nil 
	    inst._playingmusic = false
	    inst._musictask = nil
	    SetMusicLevel(inst, 1)

	    inst.SetMusicLevel = SetMusicLevel
end

return OnikiriUtil